﻿using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Legend;
using Telerik.Windows.Controls.TreeMap;
using System;
using System.Windows.Controls;

namespace sl_TreeMap_845694
{
    public static class TreeMapUtilities
    {
        public static bool GetIsLegendItemsGenerator(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsLegendItemsGeneratorProperty);
        }

        public static void SetIsLegendItemsGenerator(DependencyObject obj, bool value)
        {
            obj.SetValue(IsLegendItemsGeneratorProperty, value);
        }

        public static LegendItemCollection GetLegendItems(DependencyObject obj)
        {
            return (LegendItemCollection)obj.GetValue(LegendItemsProperty);
        }

        public static void SetLegendItems(DependencyObject obj, LegendItemCollection value)
        {
            obj.SetValue(LegendItemsProperty, value);
        }

        public static readonly DependencyProperty LegendItemsProperty = DependencyProperty.RegisterAttached(
            "LegendItems",
            typeof(LegendItemCollection),
            typeof(TreeMapUtilities),
            new PropertyMetadata(null));

        public static readonly DependencyProperty IsLegendItemsGeneratorProperty = DependencyProperty.RegisterAttached(
            "IsLegendItemsGenerator",
            typeof(bool),
            typeof(TreeMapUtilities),
            new PropertyMetadata(IsLegendItemsGeneratorChanged));

        private static void IsLegendItemsGeneratorChanged(DependencyObject target, DependencyPropertyChangedEventArgs args)
        {
            var treeMap = (RadTreeMap)target;

            if ((bool)args.NewValue)
            {
                GenerateLegendItems(treeMap);
            }
            else
            {
                treeMap.ClearValue(LegendItemsProperty);
            }
        }

        private static void GenerateLegendItems(RadTreeMap treeMap)
        {
            if (treeMap.ItemsSource == null)
            {
                treeMap.Dispatcher.BeginInvoke((Action)(() => GenerateLegendItems(treeMap)));
                return;
            }
         
            var treeItems = Telerik.Windows.Controls.ChildrenOfTypeExtensions.ChildrenOfType<RadTreeMapItem>(treeMap).Where(item => item.Name != "PART_RootItem").ToList();
            if (treeItems.Count == 0)
            {
                treeMap.Dispatcher.BeginInvoke((Action)(() => GenerateLegendItems(treeMap)));
                return;
            }

            LegendItemCollection legedItems = new LegendItemCollection();

            foreach (var item in treeItems)
            {
                var border = Telerik.Windows.Controls.ChildrenOfTypeExtensions.ChildrenOfType<Border>(item).FirstOrDefault();
                var gdpInfo = ((GdpInfo)((TreeMapData)border.DataContext).DataItem);

                legedItems.Add(new LegendItem
                {
                    Title = gdpInfo.Country,
                    MarkerFill = border.Background,
                    Presenter = item, // item.DataContext
                });
            }

            SetLegendItems(treeMap, legedItems);
        }
    }
}
